
/*
 * Created on Aug 22, 2005
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * History
 * v1.0 - Aug 22, 2005 - Initial Release
 * v1.1 - Sep 10, 2005 - Support a fully transparent image
 */

/**
 * @author kuwanger
 *
 * Contains the actualy BorderImage by its components
 */

import java.awt.image.BufferedImage;

public class BorderImage {
	// Only enough space for 90% of the screen to have unique tiles
    private final int ADDER = 0xE028;	// (Palette used)<<8 + (font start address)>>5
	private final int MAXTILES = 240;
	
	Tile Tiles[] = new Tile[MAXTILES];
	Palette pal = new Palette();
	ScreenMap Screen = new ScreenMap();
	
	int tiles = 0;
	
	BorderImage() throws TooManyTilesException, IncorrectDimensionsException, TooManyColorsException, InvalidRGBException
	{
		for (int y = 0; y < 20; y++)
		{
			for (int x = 0; x < 32; x++)
			{
				setTransparent(x, y);
			}
		}
	}
	BorderImage(BufferedImage bi) throws TooManyTilesException, IncorrectDimensionsException, TooManyColorsException, InvalidRGBException
	{
		if (bi.getWidth() != 240 ||
		    bi.getHeight() != 160)
			throw new IncorrectDimensionsException();
		for (int y = 0; y < 160; y+=8)
		{
			for (int x = 0; x < 240; x+=8)
			{
				setTile(x/8, y/8, bi.getSubimage(x, y, 8, 8));
			}
		}
	}
	
	void setTransparent(int x, int y)
	{
		Screen.setXY(x, y, (char) 0x120);
	}
	
	void setTile(int x, int y, BufferedImage bi) throws TooManyTilesException, InvalidRGBException, TooManyColorsException
	{
		if (y > 0  && y < 19 && x > 4 && x < 25)
			setTransparent(x, y);
		else {
			pal.updatePalette(bi);
			Tile temp = new Tile(bi, pal);
			for (int i = 0; i < tiles; i++) {
				if (temp.Compare(Tiles[i]))
				{
					Screen.setXY(x, y, (char) (ADDER + i));
					return;
				}
				temp.horizontal_mirror();
				if (temp.Compare(Tiles[i]))
				{
					Screen.setXY(x, y, (char) (ADDER + i | (1 << 10)));
					return;
				}
				temp.vertical_mirror();
				if (temp.Compare(Tiles[i]))
				{
					Screen.setXY(x, y, (char) (ADDER + i | (1 << 10) | (1 << 11)));
					return;
				}
				temp.horizontal_mirror();
				if (temp.Compare(Tiles[i]))
				{
					Screen.setXY(x, y, (char) (ADDER + i | (1 << 11)));
					return;
				}
				temp.vertical_mirror();
			}
			
    		if (tiles >= MAXTILES)
    			throw new TooManyTilesException();
    		
			Screen.setXY(x, y, (char) (ADDER + tiles));
			Tiles[tiles++] = temp;
		}
	}
	
	int TileCount()
	{
		return tiles;
	}
	
	Tile[] getTiles()
	{
		return Tiles;
	}
	
	byte[] getTilesBytes()
	{
		byte rawtiles[] = new byte[tiles*32];
		byte rawtile[] = new byte[32];
		for (int i = 0; i < tiles; i++)
		{
			rawtile = Tiles[i].getTileBytes();
			for (int j = 0; j < 32; j++)
			{
				rawtiles[i*32+j] = rawtile[j];
			}
		}
		
		return rawtiles;
	}
	
	char[] getPaletteChars()
	{
		return pal.getPaletteChars();
	}
	
	char [] getScreenChars()
	{
		return Screen.getScreenChars();
	}
}
