/*
 * Created on Aug 22, 2005
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * History
 * v1.0 - Aug 22, 2005 - Initial Release
 */

/**
 * @author kuwanger
 *
 * GBA Tile, using a specified palette
 */

import java.awt.image.BufferedImage;

public class Tile {
   	byte tile[][] = new byte[8][8];
	
   	void print()
   	{
   		for (int y = 0; y < 8; y++)
   		{
   			for (int x = 0; x < 8; x++)
   			{
   				System.out.print(tile[x][y] + " ");
   			}
   			System.out.println("");
   		}
		System.out.println("");
   	}
   	
   	byte[] getTileBytes()
   	{
   		byte t[] = new byte[32];
   		
   		for (int y = 0; y < 8; y++)
   		{
   			for (int x = 0; x < 8; x+=2)
   			{
   				t[y*4+(x>>1)] = (byte) ((tile[x+1][y] << 4) + tile[x][y]);
   			}
   		}
   		
   		return t;
   	}
   	
	Tile(BufferedImage bi, Palette pal) throws InvalidRGBException {
		for (int y = 0; y < 8; y++)
		{
			for (int x = 0; x < 8; x++)
			{
				setXY(x, y, (byte) (pal.getByRGB(bi.getRGB(x,y))));    				
			}
		}
	}
	
	boolean Compare(Tile t)
	{
		for (int x = 0; x < 8; x++)
		{
			for (int y = 0; y < 8; y++)
				if (tile[x][y] != t.getXY(x, y))
					return false;
		}
		return true;
	}
	
	byte getXY(int x, int y) {
		return tile[x][y];
	}
	
	void setXY(int x, int y, byte c) {
		tile[x][y] = c;
	}
	
	void horizontal_mirror() {
		byte c;
		
		for (int y = 0; y < 8; y++) {
			for (int x = 0; x < 4; x++) {
				c = getXY(x, y);
				setXY(x,y,getXY(7-x, y));
				setXY(7-x,y,c);
			}
		}
	}
	
	void vertical_mirror() {
		byte c;
		
		for (int y = 0; y < 4; y++) {
			for (int x = 0; x < 8; x++) {
				c = getXY(x, y);
				setXY(x,y,getXY(x, 7-y));
				setXY(x,7-y,c);
			}
		}
	}
}
