#!/usr/bin/python

from glob import glob
import os, sys, struct
from sys import argv
from stat import *

DGBMAXID = 0x4d424744

def readfile(name):
	try:
		fd = open(name, "rb")
		contents = fd.read()
		fd.close()
	except IOError:
		print "Error reading", name
		sys.exit(1)
	return contents

def writefile(name, contents):
	try:
		fd = open(name, "wb")
		fd.write(contents)
		fd.close()
	except IOError:
		print "Error writing", name
		sys.exit(1)

def col(color):
	c = int(color, 10)*8
	if c != 0:
		c += 7
	if c > 255:
		c = 255
	return c

def rgb(color):
	return struct.pack("BBB", col(color[0:2]), col(color[2:4]), col(color[4:6]))

def parse(file, out):
	txt = readfile(file)
	count = 0
	colors = ""
	crc = -1
	bg = -1
	win = -1
	obj = -1
	for f in txt.splitlines():
		line = f.partition(";")[0].strip()
		#print line
		if len(line) > 5 and line[0:5] == "-----":
			crc = -1
			bg = -1
			win = -1
			obj = -1
		elif len(line) == 11 and line[0:7] == "CRC: 0x":
			crc = int(line[7:], 16)
		elif len(line) == 32+10 and line[0:10] == "BGPalette:":
			bg = rgb(line[12:18]) + rgb(line[20:26]) + rgb(line[28:34]) + rgb(line[36:42])
		elif len(line) == 32+11 and line[0:11] == "WINPalette:":
			win = rgb(line[13:19]) + rgb(line[21:27]) + rgb(line[29:35]) + rgb(line[37:43])
		elif len(line) == 24+11 and line[0:11] == "OBJPalette:":
			obj = rgb(line[13:19]) + rgb(line[21:27]) + rgb(line[29:35]) + "\0\0\0"
			#obj = rgb(line[13:19]) + rgb(line[21:26]) + rgb(line[29:35]) + "\0\0\0"
		if crc != -1 and bg != -1 and win != -1 and obj != -1:
			colors += struct.pack("<H", crc) + bg + win + obj
			count += 1
			crc = -1
			bg = -1
			win = -1
			obj = -1
	if count > 0:
		dgbmax = struct.pack("<IH", DGBMAXID, count) + colors
		# make it a multiple of 4
		if len(dgbmax)&3 != 0:
			dgbmax += "\0\0"
		writefile(out, dgbmax)
		return 0
	return 1

if __name__ == "__main__":
	if len(argv) != 3:
		print "Usage: %s dgbmax.ini dgbmax.bin"
		sys.exit(0)
	if parse(argv[1], argv[2]):
		print "Unable to find any colors"
		sys.exit(1)
