/*
	GOOMBA PALETTE BUILDER - by Marc
	--------------------------------
	This tool creates a .pal file for Goomba Paletted by kuwanger
	(http://www.kuwanger.net).
	
	Usage: GOOMBA_PAL_BUILDER [file.txt]
	       (file is optional, if no file is specified it will load
	        'goomba_palettes.txt')
	
	The .txt file structure is different from old Goomba Paletted
	Python script. An example is on 'goomba_palettes.txt' that
	includes all old palettes plus new ones made by me :)
	
	The final .pal file must be appended to 'goomba_raw.gba'.

	--by Marc
	http://usuaris.tinet.cat/mark/
*/

#include <stdio.h>
#include <malloc.h>
#include <string.h>

#define DEFAULT_FILE	"goomba_palettes.txt"
#define MAX_PALETTES	256
#define MAX_TITLE		20
#define PAL_TYPES		4

typedef struct{
	unsigned char file_id[4];
	unsigned char colors[PAL_TYPES][4][3];
	unsigned char title[MAX_TITLE];
	unsigned char eof[4];
} pal;

char* substr(char* oldString,int start,int length){
	char *newString=malloc(length*(sizeof(char))+1);
	int i;
	for(i=0;i<length;i++){
		newString[i]=oldString[start+i];
	}
	newString[length]='\0';

	return newString;
}

int main(int argc, char **argv){
	unsigned int i, j, k;
	int nPals=0;
	char buffer[128];
	char filename[128];
	FILE *file;
	pal palettes[MAX_PALETTES];
	const char* palTypes[PAL_TYPES]={"bgr=", "win=", "ob1=", "ob2="};
	char* nextCh;

	printf("GOOMBA PALETTE BUILDER   (by Marc)\n");
	printf("----------------------\n");

	if(argc>1){
		strcpy(filename, argv[1]);
		if(strcmp(substr(filename, strlen(filename)-4, 4), ".txt")){
			printf("ERROR: Invalid file.\n");
			return -2;
		}
	}else{
		sprintf(filename, "goomba_palettes.txt");
	}

	//Initialize default values
	for(i=0; i<MAX_PALETTES; i++){
		palettes[i].file_id[0]=0x89;
		palettes[i].file_id[1]=0x47;
		palettes[i].file_id[2]=0x7B;
		palettes[i].file_id[3]=0x98;
		
		for(j=0; j<PAL_TYPES; j++){
			for(k=0; k<3; k++){
				palettes[i].colors[j][0][k]=0x00;
				palettes[i].colors[j][1][k]=0x50;
				palettes[i].colors[j][2][k]=0xA0;
				palettes[i].colors[j][3][k]=0xFF;
			}
		}

		for(j=0; j<MAX_TITLE; j++){
			palettes[i].title[j]=0;
		}

		for(j=0; j<4; j++){
			palettes[i].eof[j]=0;
		}
	}

	file=fopen(filename, "r");

	if(file==NULL){
		printf("ERROR: I can't open %s.\n", filename);
		return -1;
	}

	//Read file line by line
	while(fgets(buffer, sizeof buffer, file) != NULL){
		if(buffer[0]=='[' && buffer[strlen(buffer)-2]==']'){
			//A palette was found
			nPals++;
			nextCh=&buffer[1];
			for(i=0; i<MAX_TITLE && nextCh[i]!=']'; i++){
				palettes[nPals-1].title[i]=nextCh[i];
			}
		}else if(nPals>0 && nPals<MAX_PALETTES-1){
			for(i=0; i<PAL_TYPES; i++){
				if(strncmp(palTypes[i], buffer, 4)==0){
					nextCh=buffer;

					//to lowercase
					for(j=0; j<strlen(buffer); j++)
						if(buffer[j] >= 0x41 && buffer[j] <= 0x5A)
							buffer[j]=buffer[j]+0x20;


					for(j=0; j<4; j++){
						nextCh=strchr(nextCh, '#')+1;

						for(k=0; k<3; k++){
							char* clr;
							unsigned int byte;
							clr=substr(nextCh, k*2, 2);
							sscanf(clr, "%02x", &byte);
							palettes[nPals-1].colors[i][j][k]=byte;
							free(clr);
						}
					}
					//printf("%s#%02x%02x%02x #%02x%02x%02x #%02x%02x%02x #%02x%02x%02x\n", palTypes[i], palettes[nPals].colors[i][0][0], palettes[nPals].colors[i][0][1], palettes[nPals].colors[i][0][2], palettes[nPals].colors[i][1][0], palettes[nPals].colors[i][1][1], palettes[nPals].colors[i][1][2], palettes[nPals].colors[i][2][0], palettes[nPals].colors[i][2][1], palettes[nPals].colors[i][2][2], palettes[nPals].colors[i][3][0], palettes[nPals].colors[i][3][1], palettes[nPals].colors[i][3][2]);

					break;
				}
			}
			//printf("* Found palette: %s\n", palettes[nPals-1].title);
		}

	}
	fclose(file);


	filename[strlen(filename)-3]='p';
	filename[strlen(filename)-2]='a';
	filename[strlen(filename)-1]='l';

	file=fopen(filename, "wb");
	if(file==NULL){
		printf("ERROR: I can't open output file: %s\n", filename);
		return -1;
	}

	int newSize=sizeof(pal)*nPals;
	nextCh=(char*)&palettes;
	for(i=0; i<newSize; i++)
		fputc(nextCh[i], file);

	fclose(file);
	printf("%d palettes found.\n", nPals);
	printf("Created %s successfully (%d bytes).\n", filename, newSize);


	return 0;
}
